# 选择可视面.rb
# 插件名称：选择可视面
# 作者：醇青
# 版本：1.6 (支持任意选中的面，无需在组内操作)

require 'sketchup.rb'

module CameraFaceSelector
  PLUGIN_NAME = "选择可视面"
  PLUGIN_AUTHOR = "醇青"
  PLUGIN_VERSION = "1.6"
  PLUGIN_DESCRIPTION = '选择当前相机视角可见的面'.freeze
  PLUGIN_COPYRIGHT = "©#{Time.now.year} #{PLUGIN_AUTHOR}"
  
  def self.select_visible_faces
    model = Sketchup.active_model
    view = model.active_view
    camera = view.camera
    selection = model.selection
    
    # 获取所有选中的面
    selected_faces = selection.grep(Sketchup::Face)
    
    # 检查是否选中任何面
    if selected_faces.empty?
      UI.messagebox("⚠️ 未选中任何面，请选择需要筛选的面")
      return
    end
    
    # 获取相机的方向和位置
    camera_direction = camera.direction.normalize
    camera_position = camera.eye
    
    # 筛选朝向相机的面
    visible_faces = []
    
    selected_faces.each do |face|
      # 计算面所在实体的完整变换矩阵
      transform = Geom::Transformation.new
      entity = face
      
      # 沿着实体层级向上追踪，计算累加变换
      while entity && entity.respond_to?(:parent)
        if entity.respond_to?(:transformation)
          transform = transform * entity.transformation
        end
        entity = entity.parent
      end
      
      # 计算面的中心点（世界坐标）
      face_center = face.bounds.center.transform(transform)
      
      # 获取面法向量（世界坐标）
      face_normal = face.normal.clone
      face_normal.transform!(transform)
      face_normal.normalize!
      
      # 方法1：使用相机方向向量直接判断
      # 当点积 < 0 表示法向量朝向相机方向（与相机方向相反）
      dot_product1 = face_normal.dot(camera_direction)
      
      # 方法2：使用相机到面的向量判断
      vector_to_face = face_center - camera_position
      view_vector = vector_to_face.normalize
      dot_product2 = face_normal.dot(view_vector)
      
      # 双保险判断逻辑（任一方法成立即视为可见面）
      if dot_product1 < 0 || dot_product2 < 0
        visible_faces << face
      end
    end
    
    # 更新选择集
    selection.clear
    if visible_faces.any?
      selection.add(visible_faces)
      puts "✓ 选择了 #{visible_faces.size} 个朝相机方向的面"
    else
      UI.messagebox("⚠️ 未找到朝向相机的面")
    end
  rescue => e
    puts "选择可视面错误: #{e.message}"
    UI.messagebox("选择可视面时出错: #{e.message[0..40]}...")
  ensure
    model.commit_operation
  end

  # 添加右键菜单项
  @context_menu_added = false
  def self.add_context_menu
    return if @context_menu_added
    @context_menu_added = true
    
    UI.add_context_menu_handler do |menu|
      # 只要选中面就显示菜单项
      model = Sketchup.active_model
      if model.selection.grep(Sketchup::Face).any?
        menu.add_separator
        menu.add_item(PLUGIN_NAME) { 
          model.start_operation(PLUGIN_NAME, true)
          self.select_visible_faces 
        }
      end
    end
  end

  # 在插件菜单中注册
  @plugins_menu_added = false
  def self.add_to_plugins_menu
    return if @plugins_menu_added
    @plugins_menu_added = true
    
    menu_name = if UI.menu("Plugins")
                 "Plugins"
               elsif UI.menu("插件")
                 "插件"
               else
                 nil
               end
               
    if menu_name
      plugins_menu = UI.menu(menu_name)
      plugins_menu.add_item(PLUGIN_NAME) { 
        model = Sketchup.active_model
        model.start_operation(PLUGIN_NAME, true)
        self.select_visible_faces 
        model.commit_operation
      }
    end
  end

  # 插件主入口
  def self.init_plugin
    # 添加右键菜单
    self.add_context_menu
    
    # 添加到插件菜单
    self.add_to_plugins_menu
    
    # 显示加载信息
    puts "✅ #{PLUGIN_NAME} v#{PLUGIN_VERSION} 已成功加载"
    puts "使用方式："
    puts "1. 在模型中选中一个或多个面"
    puts "2. 右键选择[#{PLUGIN_NAME}]"
    puts "3. 插件会自动筛选朝向相机的面"
    puts "4. 被选择的面可直接进行右键操作（推拉、材质等）"
    puts "作者：#{PLUGIN_AUTHOR}"
  end

  # ================= 扩展注册 =================
  unless defined?(@@plugin_loaded) && @@plugin_loaded
    # 注册扩展
    extension = SketchupExtension.new(PLUGIN_NAME, File.basename(__FILE__))
    extension.description = PLUGIN_DESCRIPTION
    extension.version = PLUGIN_VERSION
    extension.creator = PLUGIN_AUTHOR
    extension.copyright = PLUGIN_COPYRIGHT
    
    Sketchup.register_extension(extension, true)
    
    # 初始化插件
    self.init_plugin
    @@plugin_loaded = true
  end
end